/*
 * libosinfo:
 *
 * Copyright (C) 2014 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __OSINFO_VERSION_H__
#define __OSINFO_VERSION_H__

/**
 * OSINFO_MAJOR_VERSION:
 *
 * libosinfo major version number.
 */
#define OSINFO_MAJOR_VERSION (1)

/**
 * OSINFO_MINOR_VERSION:
 *
 * libosinfo minor version number.
 */
#define OSINFO_MINOR_VERSION (10)

/**
 * OSINFO_MICRO_VERSION:
 *
 * libosinfo micro version number.
 */
#define OSINFO_MICRO_VERSION (0)

#endif /* __OSINFO_VERSION_H__ */
