
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

            #include <glib-object.h>


            G_BEGIN_DECLS

/* enumerations from "osinfo_install_config_param.h" */


GType osinfo_install_config_param_policy_get_type (void);
#define OSINFO_TYPE_INSTALL_CONFIG_PARAM_POLICY (osinfo_install_config_param_policy_get_type())

/* enumerations from "osinfo_install_script.h" */


GType osinfo_path_format_get_type (void);
#define OSINFO_TYPE_PATH_FORMAT (osinfo_path_format_get_type())


GType osinfo_device_driver_signing_req_get_type (void);
#define OSINFO_TYPE_DEVICE_DRIVER_SIGNING_REQ (osinfo_device_driver_signing_req_get_type())


GType osinfo_install_script_injection_method_get_type (void);
#define OSINFO_TYPE_INSTALL_SCRIPT_INJECTION_METHOD (osinfo_install_script_injection_method_get_type())


GType osinfo_install_script_installation_source_get_type (void);
#define OSINFO_TYPE_INSTALL_SCRIPT_INSTALLATION_SOURCE (osinfo_install_script_installation_source_get_type())

/* enumerations from "osinfo_loader.h" */


GType osinfo_error_get_type (void);
#define OSINFO_TYPE_ERROR (osinfo_error_get_type())

/* enumerations from "osinfo_media.h" */


GType osinfo_media_error_get_type (void);
#define OSINFO_TYPE_MEDIA_ERROR (osinfo_media_error_get_type())


GType osinfo_media_detect_flags_get_type (void);
#define OSINFO_TYPE_MEDIA_DETECT_FLAGS (osinfo_media_detect_flags_get_type())

/* enumerations from "osinfo_os.h" */


GType osinfo_release_status_get_type (void);
#define OSINFO_TYPE_RELEASE_STATUS (osinfo_release_status_get_type())

/* enumerations from "osinfo_product.h" */


GType osinfo_product_relationship_get_type (void);
#define OSINFO_TYPE_PRODUCT_RELATIONSHIP (osinfo_product_relationship_get_type())

/* enumerations from "osinfo_tree.h" */


GType osinfo_tree_error_get_type (void);
#define OSINFO_TYPE_TREE_ERROR (osinfo_tree_error_get_type())

G_END_DECLS

/* Generated data ends here */

